<?php
/**
* Shibboleth auth plug-in for phpBB3.
*
* Created from auth_apache and auth_cas.
*
* @package login
* @version 1.0
* @copyright (c) Martin Kuba
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
* http://www.ics.muni.cz/~makub/
* 
* 
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
    exit;
}

function getUserName() {
  if(empty($_SERVER['REMOTE_USER'])) return NULL;

  return $_SERVER['REMOTE_USER'];
}
function getUserEmail() {
  return $_SERVER['HTTP_SHIB_INETORGPERSON_MAIL'];
}
function getUserCommonName() {
  return $_SERVER['HTTP_SHIB_PERSON_COMMONNAME'];
}
function getUserOrg() {
  return $_SERVER['HTTP_SHIB_PERSON_O'];
}

function user_custom_profile_fields_shibboleth() 
{

        $cp_data = array();

        // DO NOT FORGET TO CREATE THE CUSTOM PROFILE FIELDS !!! 
        //$cp_data['pf_'.'realid'] = utf8_normalize_nfc(getUserCommonName().', '.getUserOrg());
        $cp_data['pf_'.'cn'] = utf8_normalize_nfc(getUserCommonName());
        $cp_data['pf_'.'org'] = utf8_normalize_nfc(getUserOrg());

        return $cp_data;
}

/**
* This function generates an array which can be passed to the user_add function in order to create a user
*/
function user_row_shibboleth($username, $password)
{
    global $db, $config, $user;
    // first retrieve default group id
    $sql = 'SELECT group_id
        FROM ' . GROUPS_TABLE . "
        WHERE group_name = '" . $db->sql_escape('REGISTERED') . "'
            AND group_type = " . GROUP_SPECIAL;
    $result = $db->sql_query($sql);
    $row = $db->sql_fetchrow($result);
    $db->sql_freeresult($result);

    if (!$row)
    {
        trigger_error('NO_GROUP');
    }

    // generate user account data
    return array(
        'username'      => $username,
        'user_password' => phpbb_hash($password),
        'user_email'    => getUserEmail(),
        'group_id'      => (int) $row['group_id'],
        'user_type'     => USER_NORMAL,
        'user_ip'       => $user->ip,
        'user_notify'   => 1,
        'user_sig'      => getUserCommonName().', '.getUserOrg(),
    );
}

/* --- no need to modify below this line -- */

/**
* Checks whether the user is identified to apache
* Only allow changing authentication to shibboleth if the user is identified
* Called in acp_board while setting authentication plugins
*
* @return boolean|string false if the user is identified and else an error message
*/
function init_shibboleth()
{
    global $user;

    if ((getUserName()==NULL) || $user->data['username'] !== getUserName())
    {
        return $user->lang['APACHE_SETUP_BEFORE_USE'];
    }
    return false;
}

/**
* Login function, copy from auth_apache with change from $_SERVER['PHP_AUTH_USER'] to getUserName().
* Called when a login form is submitted.
*/
function login_shibboleth(&$username, &$password)
{
    //error_log('login_shibboleth('.getUserName().')');
    global $db;

    if (getUserName()==NULL)
    {
        return array(
            'status'        => LOGIN_ERROR_EXTERNAL_AUTH,
            'error_msg'     => 'LOGIN_ERROR_EXTERNAL_AUTH_APACHE',
            'user_row'      => array('user_id' => ANONYMOUS),
        );
    }

    $php_auth_user = getUserName();
    $php_auth_pw = getUserName();

    if (!empty($php_auth_user) && !empty($php_auth_pw))
    {
        if ($php_auth_user !== $username)
        {
            return array(
                'status'    => LOGIN_ERROR_USERNAME,
                'error_msg' => 'LOGIN_ERROR_USERNAME',
                'user_row'  => array('user_id' => ANONYMOUS),
            );
        }

        $sql = 'SELECT user_id, username, user_password, user_passchg, user_email, user_type
            FROM ' . USERS_TABLE . "
            WHERE username = '" . $db->sql_escape($php_auth_user) . "'";
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        $db->sql_freeresult($result);

        if ($row)
        {
            // User inactive...
            if ($row['user_type'] == USER_INACTIVE || $row['user_type'] == USER_IGNORE)
            {
                return array(
                    'status'        => LOGIN_ERROR_ACTIVE,
                    'error_msg'     => 'ACTIVE_ERROR',
                    'user_row'      => $row,
                );
            }

            // Successful login...
            return array(
                'status'        => LOGIN_SUCCESS,
                'error_msg'     => false,
                'user_row'      => $row,
            );
        }

        // this is the user's first login so create an empty profile
        return array(
            'status'        => LOGIN_SUCCESS_CREATE_PROFILE,
            'error_msg'     => false,
            'user_row'      => user_row_shibboleth($php_auth_user, $php_auth_pw),
        );
    }

    // Not logged into apache
    return array(
        'status'        => LOGIN_ERROR_EXTERNAL_AUTH,
        'error_msg'     => 'LOGIN_ERROR_EXTERNAL_AUTH_APACHE',
        'user_row'      => array('user_id' => ANONYMOUS),
    );
}

/**
* Autologin function.
* Called when new session is created.
*
* @return array containing the user row or empty if no auto login should take place
*/
function autologin_shibboleth()
{
    //error_log('autologin_shibboleth('.getUserName().')');
    global $db;

    if (getUserName()==NULL)
    {
        return array();
    }

    $php_auth_user = getUserName();
    $php_auth_pw = getUserName();

    if (!empty($php_auth_user) && !empty($php_auth_pw))
    {
        set_var($php_auth_user, $php_auth_user, 'string');
        set_var($php_auth_pw, $php_auth_pw, 'string');

        $sql = 'SELECT *
            FROM ' . USERS_TABLE . "
            WHERE username = '" . $db->sql_escape($php_auth_user) . "'";
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        $db->sql_freeresult($result);

        if ($row)
        {
            return ($row['user_type'] == USER_INACTIVE || $row['user_type'] == USER_IGNORE) ? array() : $row;
        }

        if (!function_exists('user_add'))
        {
            global $phpbb_root_path, $phpEx;

            include($phpbb_root_path . 'includes/functions_user.' . $phpEx);
        }

        // create the user if he does not exist yet
        user_add(user_row_shibboleth($php_auth_user, $php_auth_pw),user_custom_profile_fields_shibboleth());

        $sql = 'SELECT *
            FROM ' . USERS_TABLE . "
            WHERE username_clean = '" . $db->sql_escape(utf8_clean_string($php_auth_user)) . "'";
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        $db->sql_freeresult($result);

        if ($row)
        {
            return $row;
        }
    }

    return array();
}

/**
* The session validation function checks whether the user is still logged in.
* Called when session exists for every page.
*
* @return boolean true if the given user is authenticated or false if the session should be closed
*/
function validate_session_shibboleth(&$user)
{
    //error_log('validate_session_shibboleth('.$user['username'].')');
    if (getUserName()==NULL)
    {
        return false;
    }

    $php_auth_user = '';
    set_var($php_auth_user, getUserName(), 'string');

    return ($php_auth_user === $user['username']) ? true : false;
}

function acp_shibboleth(&$new)
{
    global $user;

    $tpl = '

    <dl>
        <dt><label for="ldap_server">'. $user->lang['SHIBBOLETH_WAYF'] .'</label><br /><span>'. $user->lang['SHIBBOLETH_WAYF_EXPLAIN'] .'</span></dt>
        <dd><input type="text" id="shibboleth_wayf" size="40" name="config[shibboleth_wayf]" value="' . 
         (empty($new['shibboleth_wayf'])?'':$new['shibboleth_wayf']) . '" /></dd>
    </dl>
    ';

    // These are fields required in the config table
    return array(
        'tpl'       => $tpl,
        'config'    => array('shibboleth_wayf')
    );
}

?>

